<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.pkoleasing.pl
 * @since      1.0.1
 *
 * @package    Pkol
 * @subpackage Pkol/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Pkol
 * @subpackage Pkol/public
 * @author     PKO Leasing
 */

class Pkol_Public
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.1
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.1
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    private $settings_base;
    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.1
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public $site_url;
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->settings_base = 'wpt_';
        $settings = new Pkol_Admin('check');
        $this->test_url = $settings->getApiUrl();
        $this->endpoint = $settings->getEndpoint();
        $this->env = $settings->getEnv();
        $this->site_url = esc_url(get_site_url());
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.1
     */
    public function enqueue_styles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pkol_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pkol_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/pkol-public.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.1
     */
    public function enqueue_scripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Pkol_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Pkol_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/pkol-public.js', array('jquery'), $this->version, false);
    }
    public function check_rate($request)
    {
        $id = esc_html($request->get_param('id'));
        $product = wc_get_product($id);
        $qty = (int)$request->get_param('q');

       
        $shopID = sanitize_text_field(get_option($this->settings_base . 'text_id'));
        $secret = sanitize_text_field(get_option($this->settings_base . 'text_secret'));
        $enabled = sanitize_text_field(get_option($this->settings_base . 'single_checkbox'));
        $color_style = sanitize_text_field(get_option($this->settings_base . 'radio_type'));
        $size_style = sanitize_text_field(get_option($this->settings_base . 'radio_type_size'));
        $show_rates = sanitize_text_field(get_option($this->settings_base . 'radio_type_pricing'));
        $admin = new Pkol_Admin('check');
        $verify_conection = $admin->check_connection();
       
        if ($verify_conection) {
           return '';
        }
        if ($enabled !== 'on') {
            return false;
        }

        if ($shopID && $secret) {
        }
        $id = $product->get_id();

        $product_price_net = explode('&nbsp;', strip_tags(wc_price(wc_get_price_excluding_tax($product), ['decimal_separator' => '.'])))[0];
        $validate_dots = explode('.',$product_price_net);
        
        if (count($validate_dots) > 2) {
            $product_price_net = number_format($validate_dots[0].$validate_dots[1].'.'.$validate_dots[2],2, '.','');
        }
    

       


        $tax = new WC_Tax();
        $taxes = $tax->get_rates($product->get_tax_class());
        $item_rate = null;
        if (is_array($taxes) && !empty($taxes)) {
            $rates = array_shift($taxes);
            $item_rate = round(array_shift($rates));
        }
        $term_list = wp_get_post_terms($product->get_id(), 'product_cat', array('fields' => 'ids'));
        $cat_id = (int)$term_list[0];

        if ($item_rate) {
            $admin = new Pkol_Admin('check');
            $product_tax = $admin->getRate($item_rate);
        }
        $status = false;
        $test = [
            'shopId' => $shopID,
            'widgetOption' => 1,
            'totalNetValue' =>  $product_price_net,
            'uniqueItemQuantity' => 1,
            'source' => 'ITEM',
            'products' => [
                [
                    'categoryId' =>  (int)$cat_id,
                    'quantity' => 1,
                    'netValue' => $product_price_net,
                    'vatRate' => $product_tax
                ]
            ]
        ];


        $data = json_encode($test);




        //autoryzacja
        $settings_token = $shopID . ':' . $secret;
        $token = base64_encode($settings_token);
        $env_setting = ($this->env == 'dev' ? false : true);
        $this->client = new \GuzzleHttp\Client([
            'http_errors' => false,
            'verify' => $env_setting,
            'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token]
        ]);
        $price = null;
        try {
            $response = $this->client->request(
                'POST',
                $this->test_url . $this->endpoint,
                [
                    'http_errors' => false,
                    'body' => $data,
                    'timeout' => 5, // Response timeout
                    'connect_timeout' => 5, // Connection timeout
                ],
            );
            if ($response->getBody()) {
                $data = json_decode($response->getBody());

               
                if (!$data->errors && $data->validityResult == 'VALID') {
                    if ($show_rates == 'yes') {
                        $price = $data->firstInstallment->value;
                    }
                    $status = true;
                } else {
                    $status = false;
                }
            }
        } catch (\GuzzleHttp\Exception\GuzzleException $e) {
          
               return false;
         
            $price = null;
            $status = false;
            // }
        }


      

        $file = 'pko_' . $size_style . '_' . $color_style . '_leasing.' . ($size_style == 'xxl' ? 'webp' : 'svg');
        
        if ($size_style == 'big' && $color_style !== 'blue' && $show_rates == 'no') {            
            $file = 'pko_' . $size_style . '_' . $color_style . '_leasing.webp';
        }
        $disabled = false;
        $img_styles = '';
        if ($status == false) {
            $show_rates = 'no';
            $file = 'pko_disabled_' . $size_style . '_leasing.' . ($size_style == 'xxl' ? 'webp' : 'svg');
            if ($size_style == 'big' || $size_style == 'xxl') {
            $img_styles = 'border: 1px solid #E6E6E6;border-radius:4px;';
            }
        }
        $text = '';
        $class = '';
        if ($color_style !== 'blue') {
            $border = '';
            $class = 'white';
        }
        if ($show_rates == 'yes' && $price) {
            if ($color_style == 'blue') {
                $color = '#fff';
                $border = '';
            } else {
                $color = '#000';
                $border = '';
            }
            if ($size_style == 'big') {
                
                $styles = 'style="'.$border.'position:absolute;right:4px;top:17px;line-height:17px;width:70px;display:inline-block;font-size:14px;font-family:pkobp;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            } else if ($size_style == 'xxl') {
                $styles = 'style="'.$border.'position:absolute;right:5px;top:20px;line-height:17px;width:80px;display:inline-block;font-size:15px;font-family:pkobp;color:' . $color . '"';
                $file = str_replace('.webp', '_raty.webp', $file);
            } elseif ($size_style == 'medium') {
                $styles = 'style="'.$border.'position: absolute;right: -9px;top: 13px;line-height: 14px;width: 70px;display: inline-block;font-size: 12px;font-family: pkobp;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            } else {
                $styles = 'style="'.$border.'position: absolute;right: -12px;top: 11px;line-height: 12px;width: 60px;display: inline-block;font-size: 11px;font-family: pkobp;color: #000;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            }


            $text .= '<span class="pko_rate" ' . $styles . '>Rata od<br/> ' . sprintf('%0.2f', $price) . ' zł</span>';
        }


        if ($status == true) {
            echo json_encode('<br><br><div class="pkol_widget" ><div data-baseurl="' . $this->site_url . '" data-pid="' . $id . '" id="lease_click"  style="display:inline-block;position:relative;">' . $text . '<img class="'.$class.'" src="' . $this->site_url . '/wp-content/plugins/pko-leasing-online/public/images/' . $file . '" /></div></div><style>#lease_click:hover {cursor:pointer;}</style>');
        } else {
            echo json_encode('<br><br><div class="pkol_widget" ><div data-baseurl="' . $this->site_url . '" data-pid="' . $id . '"   style="display:inline-block;position:relative;">' . $text . '<img style="'.$img_styles.'"class="'.$class.'" src="' . $this->site_url . '/wp-content/plugins/pko-leasing-online/public/images/' . $file . '" /></div></div>');
    }

        exit();
    }
    public function addEndpoint()
    {
        register_rest_route('pkol/v1', '/rate/(?P<id>\d+)', array(
            'methods' => 'GET',
            'callback' => [$this, 'check_rate'],
            'permission_callback ' => '__return_true'
        ));
    }
}
